<?php

namespace UW\FCS\Attachment;

use XF\Entity\Attachment;
use XF\Mvc\Entity\Entity;
use XF\Attachment\AbstractHandler;

class Comment extends AbstractHandler
{
	public function getContainerWith()
	{
		return ['Post', 'User','Thread'];
	}

	public function canView(Attachment $attachment, Entity $container, &$error = null)
	{
		/** @var \XF\Entity\ProfilePostComment $container */
		if (!$container->canView())
		{
			return false;
		}

		return $container->canViewAttachments($error);
	}

	public function canManageAttachments(array $context, &$error = null)
	{
		$profilePost = $this->getProfilePostFromContext($context);
		return ($profilePost && $profilePost->Thread->Forum->canUploadAndManageAttachments());
	}

	public function onAttachmentDelete(Attachment $attachment, Entity $container = null)
	{
		if (!$container)
		{
			return;
		}

		/** @var \XF\Entity\ProfilePostComment $container */
		$container->attach_count--;
		$container->save();

		// TODO: phrase for attachment_deleted
		\XF::app()->logger()->logModeratorAction($this->contentType, $container, 'attachment_deleted', [], false);
	}

	public function getConstraints(array $context)
	{
		/** @var \XF\Repository\Attachment $attachRepo */
		$attachRepo = \XF::repository('XF:Attachment');

		$constraints = $attachRepo->getDefaultAttachmentConstraints();

		$profilePost = $this->getProfilePostFromContext($context);
		if ($profilePost && $profilePost->Thread->Forum->canUploadVideos())
		{
			$constraints = $attachRepo->applyVideoAttachmentConstraints($constraints);
		}

		return $constraints;
	}

	public function getContainerIdFromContext(array $context)
	{
		return isset($context['comment_id']) ? intval($context['comment_id']) : null;
	}

	public function getContext(Entity $entity = null, array $extraContext = [])
	{
		if ($entity instanceof \UW\FCS\Entity\Comment)
		{
			$extraContext['comment_id'] = $entity->comment_id;
		}
		else if ($entity instanceof \XF\Entity\Post)
		{
			$extraContext['post_id'] = $entity->post_id;
		}
		else
		{
			throw new \InvalidArgumentException("Entity must be profile post comment or profile post");
		}

		return $extraContext;
	}

	protected function getProfilePostFromContext(array $context)
	{
		$em = \XF::em();

		if (!empty($context['comment_id']))
		{
			/** @var \XF\Entity\ProfilePostComment $profilePostComment */
			$profilePostComment = $em->find('UW\FCS:Comment', intval($context['comment_id']), ['Post']);
			if (!$profilePostComment || !$profilePostComment->canView() || !$profilePostComment->canEdit())
			{
				return null;
			}

			$profilePost = $profilePostComment->Post;
		}
		else if (!empty($context['post_id']))
		{
			/** @var \XF\Entity\ProfilePost $profilePost */
			$profilePost = $em->find('XF:Post', intval($context['post_id']));
			if (!$profilePost)
			{
				return null;
			}
		}

		return $profilePost;
	}
}