<?php

namespace UW\FCS\EditHistory;

use XF\Mvc\Entity\Entity;

class Comment extends \XF\EditHistory\AbstractHandler
{
	/**
	 * @param \XF\Entity\Post $content
	 */
	public function canViewHistory(Entity $content)
	{
		return  $content->canViewHistory();
	}

	/**
	 * @param \XF\Entity\Post $content
	 */
	public function canRevertContent(Entity $content)
	{
		return $content->canEdit();
	}

	/**
	 * @param \XF\Entity\Post $content
	 */
	public function getContentTitle(Entity $content)
	{
            return 'Comment in Thread '. $content->Thread->title;
	}

	/**
	 * @param \XF\Entity\Post $content
	 */
	public function getContentText(Entity $content)
	{
		return $content->message;
	}

	public function getContentLink(Entity $content)
	{
		return \XF::app()->router('public')->buildLink('comments', $content);
	}

	/**
	 * @param \XF\Entity\Post $content
	 */
	public function getBreadcrumbs(Entity $content)
	{
		/** @var \XF\Mvc\Router $router */
		$router = \XF::app()->container('router');

		$breadcrumbs = $content->Thread->Forum->getBreadcrumbs();
		$breadcrumbs[] = [
			'value' => $content->Thread->title,
			'href' => $router->buildLink('threads', $content->Thread)
		];
		return $breadcrumbs;
	}

	/**
	 * @param \XF\Entity\Post $content
	 */
	public function revertToVersion(Entity $content, \XF\Entity\EditHistory $history, \XF\Entity\EditHistory $previous = null)
	{		
                $content->message = $history->old_text;

		if (!$previous || $previous->edit_user_id != $content->user_id)
		{
			$content->last_edit_date = 0;
		}
		else if ($previous && $previous->edit_user_id == $content->user_id)
		{
			$content->last_edit_date = $previous->edit_date;
		}

		return $content->save();
	}

	public function getHtmlFormattedContent($text, Entity $content = null)
	{
		return \XF::app()->templater()->fn('bb_code', [$text, 'post', $content]);
	}

	public function getSectionContext()
	{
		return 'forums';
	}
        
        public function getEntityWith()
	{
		$visitor = \XF::visitor();
		return ['Thread', 'Post','Thread.Forum', 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id];
	}
}