<?php

namespace UW\FCS\Extend\InlineMod\Post;

use XF\Http\Request;
use XF\Mvc\Entity\AbstractCollection;

class Delete extends XFCP_Delete {

    
    /**
     * get me comment ids
     * @param \XF\Mvc\Entity\AbstractCollection $entities
     * @param \XF\Http\Request $request
     * @return type
     */
    public function getFormOptions(AbstractCollection $entities, Request $request) {

        $ids = $this->handler->getCookieIds($request);
        $comments = [];
        foreach ($ids as $id) {
            if ($id < 0)
                $comments[] = $id;
        }
        $response = parent::getFormOptions($entities, $request);
        $response['comments'] = $comments;
        return $response;
    }

    /**
     * lets delete the comments
     * @param \XF\Mvc\Entity\AbstractCollection $entities
     * @param array $options
     * @return type
     */
    protected function applyInternal(AbstractCollection $entities, array $options) {
        $response = parent::applyInternal($entities, $options);
        if (isset($options['comments']) && $options['comments']) {

            $commentEntites = [];
            foreach ($options['comments'] as $comment) {
                $commentEntites[] = $comment * -1;
            }

            $commentEntites = \XF::finder('UW\FCS:Comment')
                    ->where('comment_id', $commentEntites)
                    ->with('Post')
                    ->with('Thread')
                    ->fetch();

            foreach ($commentEntites AS $entity) {
                $this->applyToEntityComment($entity, $options);
            }
        }

        return $response;
    }

    /**
     * delete single comment
     * @param \XF\Mvc\Entity\Entity $comment
     * @param array $options
     */
    public function applyToEntityComment(\XF\Mvc\Entity\Entity $comment, array $options) {
        if ($options['alert'] && $comment->user_id != \XF::visitor()->user_id 
                && ($options['type'] == 'soft' && $comment->status == 2)
                ) {
            $extra = array_merge([
                'title' => $comment->Thread->title,
                'prefix_id' => $comment->Thread->prefix_id,
                'link' => $this->app()->router('public')->buildLink('nopath:comments', $comment),
                'threadLink' => $this->app()->router('public')->buildLink('nopath:threads', $comment->Thread),
                'postLink' => $this->app()->router('public')->buildLink('nopath:posts', $comment->Post),
                'reason' => $options['alert_reason']
                    ], []);

            $alertRepo = \XF::app()->repository('XF:UserAlert');
            $alertRepo->alert(
                    $comment->User, $comment->user_id, $comment->User->username, 'user', $comment->user_id, "comment_delete", $extra
            );
        }

        if ($options['type'] == 'soft') {
            $comment->softDelete($options['reason']);
        } else {
            $comment->delete();
        }
    }

}
