<?php

namespace UW\FCS\Extend\Pub\Controller;

use XF\Mvc\ParameterBag;

class InlineMod extends XFCP_InlineMod {

    public function actionPerform() {
        $type = $this->filter('type', 'str');
        $handler = $this->getInlineModHandler($type);
        $ids = $handler->getCookieIds($this->request);
        $action = $this->filter('action', 'str');
        $entities = $handler->getEntities($ids);
        $actionHandler = $handler->getAction($action);

        $response = parent::actionPerform();

        $comments = [];
        foreach ($ids as $id) {
            if ($id < 0)
                $comments[] = $id;
        }
                
        if ($action == 'undelete' || $action == 'approve' || $action == 'unapprove' || $action == 'comment_move') {
            
            $correctComments = [];
            foreach ($comments as $comment) {
                $correctComments[] = $comment * -1;
            }
            $commentsEntities = $this->finder('UW\FCS:Comment')
                    ->where('comment_id', $correctComments)
                    ->with('Thread')
                    ->with('Thread.Forum')
                    ->fetch();
            
            if($action == 'comment_move')
            {
                if($entities->count() != 1)
                {
                    return $this->error(\XF::phrase('uw_select_single_post'));
                }
                if($commentsEntities->count() < 1)
                {
                    return $this->error(\XF::phrase('uw_select_comment'));
                }
                
                foreach ($commentsEntities as $comment)
                {
                    $comment->post_id = $entities->first()->post_id;
                    $comment->thread_id = $entities->first()->thread_id;
                    $comment->save();
                    $entities->first()->Thread->rebuildCounters();
                }
            }
            if($action == 'undelete')
            {            
                foreach ($commentsEntities as $comment)
                {
                    if($comment->status == 2)
                    {
                        $comment->status = 1;
                        $comment->save();
                        if($comment->DeletionLog)
                        {
                            $comment->DeletionLog->delete();                    
                        }
                    }
                }
            }
            if($action == 'approve')
            {            
                foreach ($commentsEntities as $comment)
                {
                    if($comment->status == 3)
                    {
                        $comment->status = 1;
                        $comment->save();
                    }
                }
            }
            if($action == 'unapprove')
            {            
                foreach ($commentsEntities as $comment)
                {
                    if($comment->status == 1)
                    {
                        $comment->status = 3;
                        $comment->save();
                    }
                }
            }
            
            $handler->clearCookie($this->app->response());
            
        } else {
            if ($response instanceof \XF\Mvc\Reply\View) {
                $response->setParam('comments', $comments);
            } else if ($response instanceof \XF\Mvc\Reply\Redirect && $entities->count() == 0 && $comments) {
                $confirmed = $this->filter('confirmed', 'bool');

                if ($confirmed) {
                    $options = $actionHandler->getFormOptions($entities, $this->request);

                    // either we're confirmed or we don't have a form to render
                    $actionHandler->apply($entities, $options);

                    $reply = $this->redirect($actionHandler->getReturnUrl() ? : $this->getDynamicRedirect());
                    $actionHandler->postApply($entities, $reply, $this->app->response());

                    return $reply;
                } else {
                    $correctComments = [];
                    foreach ($comments as $comment) {
                        $correctComments[] = $comment * -1;
                    }
                    $commentsEntities = $this->finder('UW\FCS:Comment')
                            ->where('comment_id', $correctComments)
                            ->with('Thread')
                            ->with('Thread.Forum')
                            ->fetch();
                    $reply = $actionHandler->renderForm($commentsEntities, $this);

                    if ($reply && $reply instanceof \XF\Mvc\Reply\AbstractReply) {
                        $reply->setParam('comments', $comments);
                        $reply->setParam('posts', $entities);
                        $reply->setParam('total', count($entities));
                        $reply->setParam('comments', $comments);
                        $reply->setParam('canHardDelete', \XF::visitor()->hasNodePermission($commentsEntities->first()->Thread->node_id, 'uw_fcs_harddelete_comment'));
                        $reply->setParam('first', $entities->first());
                        return $reply;
                    }
                }
            }
        }
        return $response;
    }

    public function actionShowActions() {
        $type = $this->filter('type', 'str');
        $handler = $this->getInlineModHandler($type);
        $ids = [];
        if($handler)
        {
            $ids = $handler->getCookieIds($this->request);
        }
        $response = parent::actionShowActions();

        if ($response instanceof \XF\Mvc\Reply\View && $handler) {
            $comments = [];
            foreach ($ids as $id) {
                if ($id < 0)
                    $comments[] = $id;
            }

            if ($comments && $response->getParam('total') == 0) {
                $entities = $this->finder('XF:Post')
                        ->with(['Thread', 'Thread.Forum', 'Thread.Forum.Node.Permissions|' . \XF::visitor()->permission_combination_id])
                        ->limit(1)
                        ->fetch();
                $actions = $handler->getActions();
                $available = [];
                if ($entities->count()) {
                    foreach ($actions AS $actionId => $action) {
                        if ($action->canApply($entities)) {
                            $available[$actionId] = $action->getTitle();
                        }
                    }
                }
                $response->setParam('actions', $available);
            }
        }
        return $response;
    }

}
