<?php

namespace UW\FCS\Extend\Pub\Controller;
use XF\Mvc\ParameterBag;

class Post extends XFCP_Post
{      
    public function actionaddcomment(ParameterBag $params)
    {
        $visitor = \XF::visitor();
                
        $this->assertPostOnly();
        $post = $this->assertViewablePost($params->post_id);
                
        if (!$visitor->hasNodePermission($post->Thread->node_id, 'uw_fcs_add_comments') || !$post->Thread->canReply())
        {
            return $this->noPermission();
        }
        
        if ($this->filter('no_captcha', 'bool')) // JS is disabled so user hasn't seen Captcha.
        {
                $this->request->set('requires_captcha', true);
                return $this->redirect($this->buildLink('posts',$post));
        }
        else if (!$this->captchaIsValid())
        {
                return $this->error(\XF::phrase('did_not_complete_the_captcha_verification_properly'));
        }
        
        $username = $visitor->username;                        
        $this->assertNotFlooding('post');
               
        $commentEntity = $this->em()->create('UW\FCS:Comment');
        $message = $this->plugin('XF:Editor')->fromInput('message');
        
        $moderate = $post->Thread->Forum->getNewContentState($post->Thread);
        
        $preparer = \XF::app()->service('XF:Message\Preparer', 'post_comment', $commentEntity);
        $preparer->enableFilter('structuredText');
        $message = $preparer->prepare($message);
        $preparer->pushEntityErrorIfInvalid($commentEntity);
        $commentEntity->embed_metadata = $preparer->getEmbedMetadata();
        
        if($moderate != 'moderated')
        {
            $commentEntity->mentionedUsers = $preparer->getMentionedUsers();
        }
        
        $values = [
            'post_id' => $post->post_id,
            'thread_id' => $post->thread_id,
            'user_id' => $visitor->user_id,
            'message' => $message,
            'username' => $username,
            'status' => $moderate == 'moderated' ? 3 : 1
        ];
        
        $commentEntity->bulkSet($values);
        $commentEntity->save();

        ////set attachments
        $inserter = $this->service('XF:Attachment\Preparer');
        $hash = $this->filter('attachment_hash', 'str');

        if($hash && $hash != "" && $hash != null) {
            $associated = $inserter->associateAttachmentsWithContent($hash, 'post_comment', $commentEntity->comment_id);
            if ($associated)
            {
                $commentEntity->fastUpdate('attach_count', $commentEntity->attach_count + $associated);
            }
        }
        ////

        // use user preferences
        $this->repository('XF:ThreadWatch')->autoWatchThread($post->Thread, $visitor, false);
        
        $this->getThreadRepo()->markThreadReadByVisitor($post->Thread, \XF::$time);
        
        $comment = $this->finder('UW\FCS:Comment')->where('comment_id',$commentEntity->comment_id)->fetchOne();
                
        $commentCount = $this->finder('UW\FCS:Comment')
                            ->where('post_id',$post->post_id)
                            ->where('status',1)
                            ->total();
        
        $viewParams = [
                'post' => $post,
                'comment' => $comment,
                'commentNumber' => $commentCount
        ];
        
        $view = $this->view('XF:Member\NewProfilePostComments', 'uw_fcs_post_comment_new', $viewParams);
        return $view;        
    }    
    /**
     * let send email to user
     * @param \XF\Entity\User $user
     * @return boolean
     */
    public function sendEmail(\XF\Entity\User $user,$comment,$also)
    {
        if (!$user->email)
        {
            return false;
        }

        $params = [
                'comment' => $comment,
                'receiver' => $user,
                'sender' => \XF::visitor(),
                'also' => $also
            ];
        
        $this->app()->mailer()->newMail()
            ->setToUser($user)
            ->setTemplate('post_comment_insert', $params)
            ->queue();
    }
    
    public function actionLoadPreviousComments(ParameterBag $params)
    {
            $post = $this->assertViewablePost($params->post_id);
            
            $limit = \XF::options()->uw_prev_comment_limit;
                        
            $visitor = \XF::visitor();
                        
            $status = [1];
                        
            if($visitor->hasNodePermission($post->Thread->node_id, 'uw_fcs_view_deleted_comme'))
            {
                $status[] = 2;
            }
            if ($visitor->hasNodePermission($post->Thread->node_id, 'uw_fcs_view_moderated'))
            {
                $status[] = 3;
            }
            $commentsFinder = $this->finder('UW\FCS:Comment')
                            ->where('post_id',$post->post_id)
                            ->with('User')
                            ->with('Thread')
                            ->where('comment_date', '<', $this->filter('before', 'uint'))
                            ->order('comment_date', 'DESC')
                            ->where('status',$status);                            
            
            if($limit)
            {
                $commentsFinder->limit($limit);
            }
            
            $comments = $commentsFinder
                            ->fetch()
                            ->reverse();
            
            $leftComments = 0;
                        
            if ($comments->count())
            {
                $firstCommentDate = $comments->first()->comment_date;

                $moreCommentsFinder = $this->finder('UW\FCS:Comment')
                        ->where('post_id',$post->post_id)
                        ->where('status',$status)                        
                        ->where('comment_date', '<', $firstCommentDate);

                $leftComments = $moreCommentsFinder->total();

                $loadMore = ($leftComments > 0);
            }
            else
            {
                $firstCommentDate = 0;
                $loadMore = false;
            }
                        
            $commentsCount = $this->finder('UW\FCS:Comment')
                            ->where('status',$status)                    
                            ->where('post_id',$post->post_id)
                            ->total();
                        
            $viewParams = [
                    'thread' => $post->Thread,
                    'leftComments' => $leftComments,
                    'commentsCount' => $commentsCount,
                    'post' => $post,
                    'comments' => $comments,
                    'firstCommentDate' => $firstCommentDate,
                    'loadMore' => $loadMore
            ];
            return $this->view('XF:ProfilePost\LoadPrevious', 'uw_more_comments', $viewParams);
    }
    
    public function actionEdit(ParameterBag $params)
    {
        $response = parent::actionEdit($params);
        if($response instanceof \XF\Mvc\Reply\View && $this->isPost())
        {            
            $post = $response->getParam('post');
            $thread = $response->getParam('thread');
            $status = [1];
            $visitor = \XF::visitor();
            
            if($visitor->hasNodePermission($thread->node_id, 'uw_fcs_view_deleted_comme'))
            {
                $status[] = 2;
            }
            
            $limit = \XF::options()->uw_limit_per_post;
            
            $commentsFinder = $this->finder('UW\FCS:Comment')
                            ->where('post_id',$post->post_id)
                            ->with('User')
                            ->where('status',$status)
                            ->order('comment_date','DESC');
            
            $commentsCount = $commentsFinder->total();
            $comments = $commentsFinder->limit($limit)->fetch()->reverse();
            
            $sortedComments = [];
            foreach ($comments as $comment)
            {
                $sortedComments[] = $comment;
            }
            
            $response->setParam('comments', $sortedComments);
            $response->setParam('commentsCount', $commentsCount);
        }
        return $response;
    }
}