<?php

namespace UW\FCS\Extend\Pub\Controller;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\ParameterBag;

class Thread extends XFCP_Thread
{      
    
    	public function actionMultiQuote()
	{
            $response = parent::actionMultiQuote();
            $quotePlugin = $this->plugin('XF:Quote');
            $quotes = $this->filter('quotes', 'json-array');           
            $quotes = $quotePlugin->prepareQuotes($quotes);

            $commentIds = [];
            $postIds = [];
            foreach ($quotes as $key => $quote)
            {
                if($key < 0)
                {
                    $commentIds[] = $key * -1;
                }
                 else {
                     $postIds[] = $key;
                }
            }
            
            if($commentIds)
            {
                $postFinder = $this->finder('XF:Post');
                $posts = $postFinder
                    ->with(['User', 'Thread', 'Thread.Forum'])
                    ->where('post_id', $postIds)
                    ->order('post_date', 'DESC')
                    ->fetch()
                    ->filterViewable();
                
                $comments = $this->finder('UW\FCS:Comment')
                        ->with('User')
                        ->where('comment_id',$commentIds)
                        ->order('comment_date', 'DESC')
                        ->fetch();
                
                foreach ( $comments as $comment)
                {
                    $posts->offsetSet(($comment->comment_id * -1) ,$comment);
                }                
                                
                if ($this->request->exists('insert'))
                {
                        $insertOrder = $this->filter('insert', 'array');
                        return $quotePlugin->actionMultiQuoteExtended($posts, $insertOrder, $quotes, 'post');
                }            
                else if ($response instanceof \XF\Mvc\Reply\View)
                {
                        $response->setParam('posts',$posts);
                }                
            }
            return $response;
	}

    protected function getNewPostsReplyInternal(
        \XF\Entity\Thread $thread,
        AbstractCollection $posts,
        \XF\Entity\Post $firstUnshownPost = null
    )
    {
        $response  = parent::getNewPostsReplyInternal($thread,$posts,$firstUnshownPost);
            /** @var \XF\Repository\Attachment $attachmentRepo */
            $attachmentRepo = $this->repository('XF:Attachment');
            $commentAttachData = [];
            foreach ($posts AS $post)
            {
                if ($post->Thread->Forum->canUploadAndManageAttachments())
                {
                    $commentAttachData[$post->post_id] = $attachmentRepo->getEditorData('post_comment', $post);
                }
            }
            $response->setParam('commentAttachData', $commentAttachData);
        return $response;
    }

    public function actionIndex(ParameterBag $params)
    {
        $response = parent::actionIndex($params);
        if($response instanceof \XF\Mvc\Reply\View)
        {
            $forum = $response->getParam('forum');
            $posts = $response->getParam('posts');
            
            $posts = is_array($posts) ? $posts : $posts->toArray();
            $postIds = array_keys($posts);
            $thread = $response->getParam('thread');
            
            $visitor = \XF::visitor();
                        
            $status = [1];
            
            if($visitor->hasNodePermission($forum->node_id, 'uw_fcs_view_deleted_comme'))
            {
                $status[] = 2;
            }
            if ($visitor->hasNodePermission($forum->node_id, 'uw_fcs_view_moderated'))
            {
                $status[] = 3;
            }
            
            $comments = $this->finder('UW\FCS:Comment')
                            ->where('post_id',$postIds)
                            ->with('User')
                            ->where('status',$status)
                            ->order('comment_date','ASC')
                            ->keyedBy('comment_id')
                            ->fetch();
         
            $targetPost = false;
            
            $gotoComment = \XF::session()->get('goto-comment');
                        
            if($gotoComment && isset($comments[$gotoComment]) && isset($comments[$gotoComment]->post_id))
            {
                $targetPost = $comments[$gotoComment]->post_id;
            }
            
            \XF::session()->set('goto-comment', 0);
            
            $comments = $comments->groupBy('post_id');
            
            $limit = \XF::options()->uw_limit_per_post;
            
            $sortedComments = [];
            $commentsCount = [];
                        
            foreach ($comments as $key => $comment)
            {
                $commentsCount[$key] = count($comment);
                if($limit == 0)
                {
                    $sortedComments[$key] = $comment;
                    continue;
                }
                $counter = 0;
                foreach ($comment as $c)
                {
                    $commentCount = count($comment) - $counter;
                    if($commentCount <= $limit || $c->isUnread() || ($targetPost && ($key == $targetPost) && $c->comment_id == $gotoComment))
                    {
                        $sortedComments[$key] = array_slice($comment, ($commentCount * -1));
                        break;
                    }
                    $counter = $counter + 1;
                }
            }

            $postsCount = $thread->reply_count + 1;

            $filters = $response->getParam('filters');
            if(\XF::options()->uw_count_comment_as_reply && !$filters)
            {
                $visiblePosts = \XF::db()->fetchOne("
			        SELECT COUNT(*)
			        FROM xf_post
			        WHERE thread_id = ?
				    AND message_state = 'visible'
		            ", $thread->thread_id);
                $postsCount = max(0, $visiblePosts);
                $response->setParam('totalPosts', max(0, $postsCount));
            }

            /** @var \XF\Repository\Attachment $attachmentRepo */
            $attachmentRepo = $this->repository('XF:Attachment');

            $commentAttachData = [];
            foreach ($posts AS $post)
            {
                if ($post->Thread->Forum->canUploadAndManageAttachments())
                {
                    $commentAttachData[$post->post_id] = $attachmentRepo->getEditorData('post_comment', $post);
                }
            }

            //for marking comments read
            $commentKeys = [];
            foreach ($sortedComments as $post) {
                foreach ($post as $comment)
                {
                    $commentKeys[] = $comment->comment_id;
                }
            }

            /** @var \XF\Repository\UserAlert $userAlertRepo */
            $userAlertRepo = $this->repository('XF:UserAlert');
            $userAlertRepo->markUserAlertsReadForContent('post_comment', $commentKeys);

            $response->setParam('postsCount', $postsCount);
            $response->setParam('comments', $sortedComments);
            $response->setParam('commentsCount', $commentsCount);
            $response->setParam('commentAttachData', $commentAttachData);
        }
        return $response;
    }
    
    public function actionUnread(ParameterBag $params)
    {
        $thread = $this->assertViewableThread($params->thread_id, ['LastPost']);
        $post = $thread->LastPost;
        
        $lastComment = $this->finder('UW\FCS:Comment')
                ->where('thread_id',$thread->thread_id)
                ->limit(1)
                ->order('comment_date','DESC')
                ->fetchOne();
        
        if($lastComment && $lastComment->canView() && $lastComment->comment_date > $post->post_date)
        {
                return $this->redirect($this->buildLink('comments', $lastComment));
        }    
        return parent::actionUnread($params);
    }
    
    public function actionLatest(ParameterBag $params)
    {
        $thread = $this->assertViewableThread($params->thread_id, ['LastPost']);
        $post = $thread->LastPost;
        
        $lastComment = $this->finder('UW\FCS:Comment')
                ->where('thread_id',$thread->thread_id)
                ->limit(1)
                ->order('comment_date','DESC')
                ->fetchOne();
        
        if($lastComment && $lastComment->canView() && $lastComment->comment_date > $post->post_date)
        {
                return $this->redirect($this->buildLink('comments', $lastComment));
        }    
        return parent::actionLatest($params);
    }
    
}