<?php

namespace UW\FCS\Extend\Service\Post;

class Copier extends XFCP_Copier {

    public function copy($sourcePostsRaw) {

        $parentResponse = parent::copy($sourcePostsRaw);

        $ids = \XF::app()->request->filter('ids', 'array-int');
        $comments = [];
        $replies = [];
        foreach ($ids as $id) {
            if ($id < 0) {
                $comment_id = ($id * -1);
                $comments[] = $comment_id;
            } else {
                $replies[] = $id;
            }
        }
                        
        if ($comments && $sourcePostsRaw->count() == 0) {
            $db = $this->db();
            $this->sourcePosts = $sourcePostsRaw;

            $target = $this->target;
            $target->setOption('log_moderator', false);

            if (!$target->thread_id) {
                $firstPost = $this->finder('UW\FCS:Comment')
                        ->where('comment_id', $comments)
                        ->with('User')
                        ->order('comment_date','desc')
                        ->fetchOne();
                                
                $target->user_id = $firstPost->user_id;
                $target->username = $firstPost->username;
                $target->post_date = $firstPost->post_date;
            }

            $db->beginTransaction();

            $target->save();

            $this->copyDataToTarget();
            $this->updateTargetData();

            if ($this->alert) {
                $this->sendAlert();
            }

            $db->commit();

            return true;
        }

        return $parentResponse;
    }

    protected function copyDataToTarget() {
        $resetValues = $this->getResetPostData();

        $position = 0;
        $firstPost = null;

        $ids = \XF::app()->request->filter('ids', 'array-int');
        $comments = [];
        $replies = [];
        foreach ($ids as $id) {
            if ($id < 0) {
                $comment_id = ($id * -1);
                $comments[] = $comment_id;
            } else {
                $replies[] = $id;
            }
        }

        $comments = $this->finder('UW\FCS:Comment')
                ->where('comment_id', $comments)
                ->with('User')
                ->fetch();
        
        // posts are sorted in date order
        foreach ($this->sourcePosts AS $sourcePost) {
            /** @var \XF\Entity\Post $newPost */
            $newPost = $this->em()->create('XF:Post');

            $values = $sourcePost->toArray(false);
            foreach ($resetValues AS $key) {
                unset($values[$key]);
            }

            $newPost->thread_id = $this->target->thread_id;
            $newPost->bulkSet($values);

            $newPost->position = $position;
            if (!$firstPost) {
                    // first post is always visible, set $firstPost later
                    $this->target->discussion_state = $newPost->message_state;
                    $newPost->message_state = 'visible';                
            }

            $newPost->save();
            //lets create comments if needed
            foreach ($comments as $key => $comment) {
                if ($comment->post_id == $sourcePost->post_id) {
                    $this->createComment($comment, $newPost->thread_id, $newPost->post_id);
                    $comments->offsetUnset($key);
                }
            }
            ///////////end here

            if (!$firstPost) {
                $firstPost = $newPost;

                $this->target->fastUpdate('first_post_id', $newPost->post_id);
            }

            $embedMetadata = $sourcePost->embed_metadata;
            $newPost->embed_metadata = $this->updateEmbeds($sourcePost, $newPost, $embedMetadata ? : []);
            $newPost->saveIfChanged();

            if ($newPost->message_state == 'visible') {
                $position++;
            }
        }
        if ($comments) {
            $this->createPosts($comments, $this->target->thread_id);
        }
    }

    /**
     * create posts
     * @param type $comment
     * @param type $thread_id
     */
    protected function createPosts($comments, $thread_id) {
        foreach ($comments as $comment) {
            if ($comment) {
                $postEntity = $this->em()->create('XF:Post');
                $values = [
                    'message' => $comment->message,
                    'thread_id' => $thread_id,
                    'user_id' => $comment->user_id,
                    'username' => $comment->username,
                    'post_date' => $comment->comment_date,
                    'position' => 0
                ];

                $postEntity->bulkSet($values);
                $postEntity->save();
            }
        }
    }

    /**
     * create posts
     * @param type $comment
     * @param type $thread_id
     */
    protected function createComment($comment, $thread_id, $post_id) {

        $commentEntity = $this->em()->create('UW\FCS:Comment');
        $values = [
            'message' => $comment->message,
            'thread_id' => $thread_id,
            'user_id' => $comment->user_id,
            'post_id' => $post_id,
            'comment_date' => $comment->comment_date,
            'username' => $comment->username
        ];

        $commentEntity->bulkSet($values);
        $commentEntity->save();
    }

}
