<?php

namespace UW\FCS\Extend\Service\Post;

class Mover extends XFCP_Mover
{
    
    public function move($sourcePostsRaw)
    {
        $ids = \XF::app()->request->filter('ids','array-int');
        
        $comments = [];
        $replies = [];
        foreach ($ids as $id)
        {
            if($id < 0)
            {
                $comment_id = ($id * -1);
                $comments[] = $comment_id; 
            }
            else
            {
                $replies[] = $id;
            }
        }
        
        if($comments && !$replies)
        {
            $db = $this->db();

            /** @var Post[] $sourcePosts */
            /** @var Thread[] $sourceThreads */
            $sourcePosts = [];
            $sourceThreads = [];

            $target = $this->target;
            $sourcePostsRaw = $this->finder('UW\FCS:Comment')
                            ->where('comment_id',$comments)
                            ->with('User')
                            ->with('Thread')
                            ->fetch();

            $sourceThread = false;
            
            foreach ($sourcePostsRaw AS $sourcePost)
            {
                    $sourcePosts[$sourcePost->comment_id] = $sourcePost;

                    /** @var Thread $sourceThread */
                    $sourceThread = $sourcePost->Thread;
                    if ($sourceThread && !isset($sourceThreads[$sourceThread->thread_id]))
                    {
                            $sourceThread->setOption('log_moderator', false);
                            $sourceThreads[$sourceThread->thread_id] = $sourceThread;
                    }
            }


            $sourcePosts = \XF\Util\Arr::columnSort($sourcePosts, 'comment_date');

            $this->sourceThreads = $sourceThreads;
            $this->sourcePosts = [];

            $target->setOption('log_moderator', false);

            if (!$target->thread_id)
            {
                    $firstPost = reset($sourcePosts);

                    $target->user_id = $firstPost->user_id;
                    $target->username = $firstPost->username;
                    $target->post_date = $firstPost->comment_date;
            }

            $db->beginTransaction();

            $target->save();

            if($sourcePosts)
            {
                $this->sourcePosts = $this->createPosts($sourcePosts, $target->thread_id);
            }
            
            $this->updateTargetData();
            $this->updateSourceData();
            $this->updateUserCounters();

            if ($this->alert)
            {
                $target = $this->target;

		/** @var \XF\Repository\Post $postRepo */
		$postRepo = $this->repository('XF:Post');

		foreach ($this->sourcePosts AS $sourcePost)
		{
                    if ($sourcePost->Thread->discussion_state == 'visible'
                            && $sourcePost->message_state == 'visible'
                            && $sourcePost->user_id != \XF::visitor()->user_id
                    )
                    {
                            $targetPost = clone $sourcePost;
                            $targetPost->setAsSaved('thread_id', $target->thread_id);

                            $alertExtras = [
                                    'comment' => 1,
                                    'sourceTitle' => $sourceThread? $sourceThread->title : $sourcePost->Thread->title,
                                    'targetLink' => $this->app->router('public')->buildLink('nopath:posts', $sourcePost)
                            ];

                            $postRepo->sendModeratorActionAlert($targetPost, 'move', $this->alertReason, $alertExtras);
                    }
		}
            }
            
            $this->finalActions();

            $db->commit();

            return true;

        }
        return parent::move($sourcePostsRaw);
    }
    
    
    /**
     * lets move the replies with thread
     * @return type
     */
    protected function moveDataToTarget()
    {
        $ids = \XF::app()->request->filter('ids','array-int');
        
        $targetExsist = $this->existingTarget;
        $sourcePostIds = array_keys($this->sourcePosts);
        $target = $this->target;
        $response = parent::moveDataToTarget();
        //lets move the comments along
        $comments = [];
        foreach ($ids as $id)
        {
            if($id < 0)
            {
                $comment_id = ($id * -1);
                $comments[] = $comment_id; 
            }
        }
        if($comments)
        {
            $comments = $this->finder('UW\FCS:Comment')
                ->where('comment_id',$comments)
                ->with('User')
                ->fetch();
            
            $this->createPosts($comments, $target->thread_id);
        }
        if($targetExsist)
        {
            $db = $this->db();
            $sourceIdsQuoted = $db->quote($sourcePostIds);

            $db->update('xf_uw_comment',
                    ['thread_id' => $target->thread_id],
                    "post_id IN ($sourceIdsQuoted)"
            );
        }
        else
        {
            $comments = $this->finder('UW\FCS:Comment')
                        ->where('post_id',$sourcePostIds)
                        ->with('User')
                        ->fetch();
            
            $this->createPosts($comments, $target->thread_id);
        }
        return $response;
    }
    /**
     * create posts
     * @param type $comment
     * @param type $thread_id
     */
    protected function createPosts($comments,$thread_id)
    {
        $posts = [];
        
        foreach ($comments as $comment)
        {
            if($comment)
            {
                $this->createPosts($comment, $thread_id);

                $postEntity = $this->em()->create('XF:Post');
                $values = [
                    'message' => preg_replace('/\[CQ=.*\[\/CQ]/', '', $comment->message),
                    'thread_id' => $thread_id,
                    'user_id' => $comment->user_id,
                    'username' => $comment->username,
                    'post_date' => $comment->comment_date,
                    'position' => 0,
                    'reaction_score' => $comment->reaction_score,
                    'reactions' => $comment->reactions,
                    'reaction_users' => $comment->reaction_users,
                ];

                $postEntity->bulkSet($values);
                $postEntity->save();
                
                $values = [
                    'content_type' => 'post',
                    'content_id' => $postEntity->post_id
                    ];
                
                $where = "content_type = 'post_comment' AND content_id = ".$comment->comment_id;
                
                \XF::db()->update('xf_reaction_content', $values, $where);
                $comment->delete();
                
                $posts[] = $postEntity;
            }
        }
        return $posts;
    }
}
