<?php

namespace UW\FCS\Pub\Controller;

use XF\Mvc\ParameterBag;

class Comment extends \XF\Pub\Controller\AbstractController
{    
    /**
     * view this comment
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actionindex(ParameterBag $params)
    {        
        $visitor = \XF::visitor();
        $extraWith[] = 'Thread.Forum';
        $extraWith[] = 'Thread.Forum.Node';
        $extraWith[] = 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id;
        $comment = $this->assertViewableComment($params->comment_id,$extraWith);
                
        \XF::session()->set('goto-comment', $comment->comment_id);
        
        $this->repository('XF:Thread')->markThreadReadByVisitor($comment->Thread, \XF::$time);
        return $this->redirectPermanently($this->getCommentLink($comment));
    }
    
    /**
     * @param $postId
     * @param array $extraWith
     * @return \XF\Entity\Post
     * @throws \XF\Mvc\Reply\Exception
     */
    protected function assertViewableComment($commentId, array $extraWith = [])
    {
        $extraWith[] = 'Thread';
        $extraWith[] = 'Post';
        
        $comment = $this->em()->find('UW\FCS:Comment', $commentId, $extraWith);

        if (!$comment || !$comment->canView())
        {
            throw $this->exception($this->notFound(\XF::phraseDeferred('requested_comment_not_found')));
        }
        
        return $comment;
    }
    
    public function actionShow(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);

        $viewParams = [
                'comment' => $comment,
                'post' => $comment->Post
        ];
        
        return $this->view('XF:Comment\Show', 'uw_fcs_post_comment_new', $viewParams);
    }
    
    /**
     * report this comment
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actionReport(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        if (!\XF::visitor()->hasNodePermission($comment->Thread->node_id, 'uw_fcs_report_comment'))
        {
            return $this->noPermission();
        }

        /** @var \XF\ControllerPlugin\Report $reportPlugin */
        $reportPlugin = $this->plugin('XF:Report');
        return $reportPlugin->actionReport(
                'post_comment', $comment,
                $this->buildLink('comments/report', $comment),
                $this->buildLink('comments', $comment)
        );
    }
    
    /**
     * get the targeted comment link
     * @param \UW\FCS\Entity\Comment $comment
     * @return type
     * @throws \LogicException
     */
    public function getCommentLink(\UW\FCS\Entity\Comment $comment)
    {
        $thread = $comment->Thread;
        if (!$thread)
        {
            throw new \LogicException("Post has no thread");
        }
        $page = floor($comment->Post->position / $this->options()->messagesPerPage) + 1;
        return $this->buildLink('threads', $thread, ['page' => $page]) . '#comment-' . $comment->comment_id;
    }
    
    public function actionhistory(ParameterBag $params)
    {
        return $this->rerouteController('XF:EditHistory', 'index', [
                'content_type' => 'post_comment',
                'content_id' => $params->comment_id
        ]);
    }

        /**
     * lets edit the comment
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actionedit(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        if (!$comment->canEdit())
        {
            return $this->noPermission();
        }

        $attachmentRepo = $this->repository('XF:Attachment');

        if ($this->isPost())
        {
            $message = $this->plugin('XF:Editor')->fromInput('message');
            $oldMessage = $comment->message;
            
            $preparer = \XF::app()->service('XF:Message\Preparer', 'post_comment', $comment);
            $preparer->enableFilter('structuredText');
            $message = $preparer->prepare($message);
            
            $comment->message = $message;
            
            $input = $this->filter([
			'silent' => 'bool',
			'clear_edit' => 'bool',
		]);
                        
            $delay =  (\XF::options()->editLogDisplay['delay'] * 60) + $comment->comment_date;

            if (!$input['silent'] && $delay <= \XF::$time)
            {
                $comment->last_edit_date = \XF::$time;
                $comment->edit_count = $comment->edit_count + 1;
                
                if ($oldMessage)
                {
                        $repo = $this->repository('XF:EditHistory');
                        $repo->insertEditHistory('post_comment', $comment, \XF::visitor(), $oldMessage, $this->app->request()->getIp());
                }
            }
            else if($input['silent'] && $comment->canEditSilently())
            {
                if ($input['clear_edit'])
                {
                    $comment->last_edit_date = 0;
                    $comment->edit_count = 0;
                }   
            }
            
            $comment->save();

            /** @var \XF\Service\Attachment\Preparer $inserter */
            $inserter = $this->service('XF:Attachment\Preparer');
            $hash = $this->filter('attachment_hash', 'str');
            $associated = $inserter->associateAttachmentsWithContent($hash, 'post_comment', $comment->comment_id);
            if ($associated)
            {
                $comment->fastUpdate('attach_count', $comment->attach_count + $associated);
            }

            if ($this->filter('_xfWithData', 'bool') && $this->filter('_xfInlineEdit', 'bool'))
            {
                $viewParams = [
                    'comment' => $comment,
                    'post' => $comment->Post
                ];
                
                $reply = $this->view('XF:ProfilePost\Comments\EditNewComment', 'uw_fcs_post_comment_new', $viewParams);
                $reply->setJsonParam('message', \XF::phrase('your_changes_have_been_saved'));
                return $reply;
            }
            else
            {
                return $this->redirect($this->buildLink('comments', $comment));
            }
        }
        else
        {
            $thread = $comment->Thread;

            if ($thread->Forum->canUploadAndManageAttachments())
            {
                $attachmentData = $attachmentRepo->getEditorData('post_comment', $comment);
            }
            else
            {
                $attachmentData = null;
            }

            $viewParams = [
                'attachmentData' => $attachmentData,
                'comment' => $comment,
                'quickEdit' => $this->responseType() == 'json'
            ];

            return $this->view('XF:ProfilePost\Comments\Edit', 'uw_fcs_post_comment_edit', $viewParams);
        }
    }
    
    /**
     * 
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actionundelete(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        if (!$comment->canDelete())
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            $comment->status = 1;
            $comment->save();
            $comment->DeletionLog->delete();
            
            if($comment->Post->Thread->last_post_date < $comment->comment_date)
            {
                $comment->Post->Thread->last_post_date = $comment->comment_date;
                $comment->Post->Thread->last_post_user_id = $comment->user_id;
                $comment->Post->Thread->save();
            }
            return $this->redirect($this->buildLink('comments', $comment));
        }
        
        $viewParams = [
                'content' => $comment,
                'confirmUrl' => $this->buildLink('comments/undelete', $comment),
                'contentUrl' => $this->buildLink('comments', $comment),
                'contentTitle' => "Comment in thread '".$comment->Thread->title."'"
        ];
        
        return $this->view('XF:Undelete\Undelete', 'public:undelete_confirm', $viewParams);
    }

    

    /**
     * delete the comment
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actiondelete(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        if (!$comment->canDelete())
        {
            return $this->noPermission();
        }

        if ($this->isPost())
        {
            $type = $this->filter('hard_delete', 'bool') ? 'hard' : 'soft';
            $reason = $this->filter('reason', 'str');
           
            if ($this->filter('author_alert', 'bool'))
            {
                $extra = array_merge([
			'title' => $comment->Thread->title,
			'prefix_id' => $comment->Thread->prefix_id,
			'link' => $this->app()->router('public')->buildLink('nopath:comments', $comment),
			'threadLink' => $this->app()->router('public')->buildLink('nopath:threads', $comment->Thread),
			'postLink' => $this->app()->router('public')->buildLink('nopath:posts', $comment->Post),
			'reason' => $this->filter('author_alert_reason', 'str')
		], []);
                
		$alertRepo = $this->repository('XF:UserAlert');
		$alertRepo->alert(
			$comment->User,
			$comment->user_id, $comment->User->username,
			'user', $comment->user_id,
			"comment_delete", $extra
		);
            }
            
            $redirtLink = $this->buildLink('posts', $comment->Post);
            
            if($type == 'soft')
            {
                if($comment->canViewDeletedComments()) {
                    $redirtLink = $this->buildLink('comments', $comment);
                }
                $comment->softDelete($reason);
            }
            else
            {
                $comment->delete();
            }
            
            $comment->Post->Thread->last_post_date = $comment->Post->Thread->LastPost->post_date;
            $comment->Post->Thread->last_post_user_id = $comment->Post->Thread->LastPost->user_id;
            $comment->Post->Thread->save();
            
            $alertRepo = $this->repository('XF:UserAlert');
            $alertRepo->fastDeleteAlertsForContent('post_comment', $comment->comment_id);
            
            return $this->redirect($redirtLink);
        }
        else
        {
            $viewParams = [
                'comment' => $comment,
                'confirmUrl' => $this->buildLink('comments/delete',$comment),
                'contentUrl' => $this->buildLink('comments',$comment),
                'contentTitle' => "Comment",
            ];

            return $this->view('XF:Mvc\View', 'uw_comment_delete_confirm', $viewParams);
        }
    }
    
    public function actionquote(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        
        return $this->plugin('XF:Quote')->actionQuote($comment, 'post');
    }

    public function actionShare(ParameterBag $params)
    {
            $comment = $this->assertViewableComment($params->comment_id);
            $thread = $comment->Thread;

            /** @var \XF\ControllerPlugin\Share $sharePlugin */
            $sharePlugin = $this->plugin('XF:Share');
            return $sharePlugin->actionTooltip(
                    $this->buildLink('canonical:comments', $comment),
                    'comment in thread'. $thread->title,
                    'Share this comment'
            );
    }
    
    public function actionReactions(ParameterBag $params)
    {
            $comment = $this->assertViewableComment($params->comment_id);

            $breadcrumbs = $comment->Thread->getBreadcrumbs();
            $title = "Members who reacted to comment";

            /** @var \XF\ControllerPlugin\Reaction $reactionPlugin */
            $reactionPlugin = $this->plugin('XF:Reaction');
            return $reactionPlugin->actionReactions(
                    $comment,
                    'comments/reactions',
                    $title, $breadcrumbs
            );
    }
    
    /**
     * react on comments
     * @param \XF\Mvc\ParameterBag $params
     * @return type
     */
    public function actionReact(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
        /** @var \XF\ControllerPlugin\Reaction $reactionPlugin */
        $reactionPlugin = $this->plugin('XF:Reaction');
        return $reactionPlugin->actionReactSimple($comment, 'comment');
    }
    
    public function actionWarn(ParameterBag $params)
    {
            $comment = $this->assertViewableComment($params->comment_id);

            if (!$comment->canWarn($error))
            {
                    return $this->noPermission($error);
            }

            $breadcrumbs = $comment->Thread->getBreadcrumbs();

            /** @var \XF\ControllerPlugin\Warn $warnPlugin */
            $warnPlugin = $this->plugin('XF:Warn');
            return $warnPlugin->actionWarn(
                    'post_comment', $comment,
                    $this->buildLink('comments/warn', $comment),
                    $breadcrumbs
            );
    }
    
    public function actionIp(ParameterBag $params)
    {
            $comment = $this->assertViewableComment($params->comment_id);
            $breadcrumbs = $comment->Thread->getBreadcrumbs();

            /** @var \XF\ControllerPlugin\Ip $ipPlugin */
            $ipPlugin = $this->plugin('XF:Ip');
            return $ipPlugin->actionIp($comment, $breadcrumbs);
    }
    
    public function actionCommentQuote(ParameterBag $params)
    {
        $comment = $this->assertViewableComment($params->comment_id);
       
        $link = $this->buildLink('goto/comment',null,['id' =>$params->comment_id]);
        if($comment->user_id)
        {
            $quote = "[CQ='".$comment->username.", c: $params->comment_id, m: $comment->user_id'][/CQ]";
            $quoteHtml = "<div>[CQ='".$comment->username.", c: $params->comment_id, m: $comment->user_id'][/CQ] </div>";
        }
        else
        {
            $quote = "[CQ='".$comment->username.", c: $params->comment_id, m: $comment->user_id'][/CQ]";
            $quoteHtml = "<div>[CQ='".$comment->username.", c: $params->comment_id, m: $comment->user_id'][/CQ] </div>";
        }
        
        $view = $this->view('XF:Quote\Quote');
        $view->setJsonParams([
                'quote' => $quote,
                'quoteHtml' => $quoteHtml
        ]);
        return $view;
    }
          
}