<?php

namespace UW\FCS\Report;

use XF\Entity\Report;
use XF\Mvc\Entity\Entity;

class Comment extends \XF\Report\AbstractHandler
{
	protected function canViewContent(Report $report)
	{
            return TRUE;
	}

	protected function canActionContent(Report $report)
	{
		$visitor = \XF::visitor();
		return ($visitor->is_admin || $visitor->is_moderator);
	}

	public function setupReportEntityContent(Report $report, Entity $content)
	{            
		/** @var \XF\Entity\Post $content */
		$report->content_user_id = $content->user_id;
		$report->content_info = [
			'message' => $content->message,			
			'user' => [
				'user_id' => $content->user_id,
				'username' => $content->User->username
			],
			'post_id' => $content->post_id,
			'comment_id' => $content->comment_id,
			'thread_id' => $content->thread_id,
		];
	}

	public function getContentTitle(Report $report)
	{
		if (isset($report->content_info['user']))
		{
			return \XF::phrase('uw_fcs_post_comment_by_x', [
				'username' => $report->content_info['user']['username']
			]);
		}
		else
		{
			return \XF::phrase('uw_fcs_post_comment_by_x', [
				'username' => $report->content_info['username']
			]);
		}
	}

	public function getContentMessage(Report $report)
	{
		return $report->content_info['message'];
	}

	public function getContentLink(Report $report)
	{
		if (isset($report->content_info['user']))
		{
			$linkData = $report->content_info;
		}
		else
		{
			$linkData = ['comment_id' => $report->content_id];
		}

		return \XF::app()->router('public')->buildLink('canonical:comments', $linkData);
	}
}