<?php

namespace UW\FCS\Search\Data;

use XF\Mvc\Entity\Entity;
use XF\Search\IndexRecord;
use XF\Search\MetadataStructure;
use XF\Search\Data\AbstractData;

class Comment extends AbstractData
{
	public function getEntityWith($forView = false)
	{
		$get = ['Post','Thread','Thread.Forum'];
		if ($forView)
		{
			$visitor = \XF::visitor();
			$get[] = 'Thread.Forum.Node.Permissions|' . $visitor->permission_combination_id;
			$get[] = 'User';
		}

		return $get;
	}

	public function getIndexData(Entity $entity)
	{
		/** @var \XF\Entity\ProfilePostComment $entity */

                if (!$entity->Post || !$entity->Thread || !$entity->Thread->Forum)
		{
			return null;
		}
		
		$index = IndexRecord::create('post_comment', $entity->comment_id, [
			'title' => '',
			'message' => $entity->message_,
			'date' => $entity->comment_date,
			'user_id' => $entity->user_id,
			'discussion_id' => $entity->Thread->thread_id,
			'metadata' => $this->getMetaData($entity)
		]);

		if (!$entity->isVisible())
		{
			$index->setHidden();
		}

		return $index;
	}

        protected function getMetaData(\UW\FCS\Entity\Comment $entity)
	{
		/** @var \XF\Entity\Thread $thread */
		$thread = $entity->Thread;

		$metadata = [
			'node' => $thread->node_id,
			'thread' => $entity->thread_id,
			'post' => $entity->post_id
		];
		
		return $metadata;
	}

	public function setupMetadataStructure(MetadataStructure $structure)
	{
		$structure->addField('node', MetadataStructure::INT);
		$structure->addField('thread', MetadataStructure::INT);
		$structure->addField('post', MetadataStructure::INT);
	}
        
	public function getResultDate(Entity $entity)
	{
		return $entity->comment_date;
	}

	public function getTemplateData(Entity $entity, array $options = [])
	{
		return [
			'comment' => $entity,
			'options' => $options
		];
	}
}