<?php

namespace UW\FCS\Service\Comment;

use UW\FCS\Entity\Comment;
use XF\Entity\Post;
use XF\Service\AbstractNotifier;

class Notifier extends AbstractNotifier
{
	/**
	 * @var Post
	 */
	protected $comment;

	protected $actionType;

	public function __construct(\XF\App $app, Comment $comment, $actionType = 'comment')
	{
		parent::__construct($app);
                
//		switch ($actionType)
//		{
//			case 'reply':
//			case 'thread':
//				break;

//			default:
//				throw new \InvalidArgumentException("Unknown action type '$actionType'");
//		}

		$this->actionType = $actionType;
		$this->comment = $comment;
	}

	public static function createForJob(array $extraData)
	{
		$post = \XF::app()->find('UW\FCS:Comment', $extraData['commentId'], ['Post','Post.Thread']);
		if (!$post)
		{
			return null;
		}
                return \XF::service('UW\FCS\Service\Comment\Notifier', $post, $extraData['actionType']);
	}

	protected function getExtraJobData()
	{
		return [
			'commentId' => $this->comment->comment_id,
			'actionType' => $this->actionType
		];
	}

	protected function loadNotifiers()
	{
		$notifiers = [
//			'quote' => $this->app->notifier('XF:Post\Quote', $this->post),
//			'forumWatch' => $this->app->notifier('XF:Post\ForumWatch', $this->post, $this->actionType),                    
			'mention' => $this->app->notifier('UW\FCS\Notifier\Comment\Mention', $this->comment),
                        'threadWatch' => $this->app->notifier('UW\FCS\Notifier\Comment\ThreadWatch', $this->comment, $this->actionType)
		];
                
		return $notifiers;
	}

	protected function loadExtraUserData(array $users)
	{
		$permCombinationIds = [];
		foreach ($users AS $user)
		{
			$id = $user->permission_combination_id;
			$permCombinationIds[$id] = $id;
		}

		$this->app->permissionCache()->cacheMultipleContentPermsForContent(
			$permCombinationIds,
			'node', $this->comment->Post->Thread->node_id
		);
	}

	protected function canUserViewContent(\XF\Entity\User $user)
	{
		return \XF::asVisitor(
			$user,
			function() { return $this->comment->canView(); }
		);
	}

	public function skipUsersWatchingForum(\XF\Entity\Forum $forum)
	{
		$watchers = $this->db()->fetchAll("
			SELECT user_id, send_alert, send_email
			FROM xf_forum_watch
			WHERE node_id = ?
				AND (send_alert = 1 OR send_email = 1)
		", $forum->node_id);

		foreach ($watchers AS $watcher)
		{
			if ($watcher['send_alert'])
			{
				$this->setUserAsAlerted($watcher['user_id']);
			}
			if ($watcher['send_email'])
			{
				$this->setUserAsEmailed($watcher['user_id']);
			}
		}
	}
}