<?php

namespace UW\FCS;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;
use XF\Db\Schema\Create;
use XF\Db\Schema\Alter;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;
        
        /**
         * create table to slider
         */
        public function installStep1()
        {
            $this->schemaManager()->createTable('xf_uw_comment', function(Create $table)
            {
                $table->addColumn('comment_id', 'int')->autoIncrement();
                $table->addColumn('message', 'mediumtext');
                $table->addColumn('comment_date', 'int');
                $table->addColumn('thread_id', 'int');
                $table->addColumn('post_id', 'int');
                $table->addColumn('user_id', 'int');
                $table->addColumn('last_edit_date', 'int');
                $table->addColumn('edit_count', 'int');
                $table->addColumn('reaction_score', 'int')->unsigned(false)->setDefault(0);
                $table->addColumn('reactions', 'blob')->nullable();
                $table->addColumn('reaction_users', 'blob');                               
                $table->addColumn('status', 'TINYINT')->setDefault(1);
                $table->addColumn('warning_id', 'int');
                $table->addColumn('warning_message', 'varchar',255);
                $table->addColumn('ip_id', 'int');
                $table->addColumn('username', 'varchar',50);
                $table->addColumn('attach_count', 'int')->setDefault(0);
                $table->addColumn('embed_metadata', 'blob')->nullable();
            });
            
            $this->schemaManager()->alterTable('xf_user_option', function(Alter $table) {
                $table->addColumn('comment_notif_all', 'tinyint', 3)->setDefault(0);
            });
            
        }
        public function upgrade9Step1()
	{
            $sm = $this->schemaManager();
		
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->addColumn('last_edit_date', 'int');
            });
        }
        public function upgrade10Step1()
	{
            $sm = $this->schemaManager();
		
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->addColumn('edit_count', 'int');
            });
        }
        
        public function upgrade16Step1()
        {
            $sm = $this->schemaManager();
		
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->addColumn('reaction_score', 'int')->unsigned(false)->setDefault(0);
                $table->addColumn('reactions', 'blob')->nullable();
                $table->addColumn('reaction_users', 'blob');               
            });
        }
        public function upgrade33Step1()
        {
            $sm = $this->schemaManager();
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->addColumn('warning_id', 'int');
                $table->addColumn('warning_message', 'varchar',255);
                $table->addColumn('ip_id', 'int');               
            });
        }
        
        public function upgrade43Step1()
        {
            $sm = $this->schemaManager();
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->addColumn('username', 'varchar',50);
            });
        }
        
        public function upgrade90Step1()
        {
            //fix for reactions count
            \XF::db()->query(
                "DELETE FROM xf_reaction_content WHERE reaction_content_id IN ( 
                SELECT * FROM ( SELECT reaction_content_id FROM `xf_reaction_content` as content 
                LEFT JOIN xf_uw_comment as commnet on (commnet.comment_id = content.content_id)
                WHERE content_type = 'post_comment' AND commnet.comment_id IS NULL) as t)");
            
            \XF::db()->query(
                "UPDATE xf_reaction_content set is_counted = 0 WHERE reaction_content_id IN (
                SELECT * FROM ( SELECT reaction_content_id FROM `xf_reaction_content` as content 
                LEFT JOIN xf_uw_comment as commnet on (commnet.comment_id = content.content_id)
                WHERE content_type = 'post_comment' AND commnet.status != 1 AND is_counted = 1 )as t)");
        }
        
        public function upgrade93Step1()
        {
            $sm = $this->schemaManager();
            $sm->alterTable('xf_uw_comment', function(Alter $table)
            {
                $table->changeColumn('reaction_score', 'int')->unsigned(false)->setDefault(0);
            });
        }
        
        public function upgrade103Step1()
        {   
            $sm = $this->schemaManager();
            $sm->alterTable('xf_user_option', function(Alter $table) {
                $table->addColumn('comment_notif_all', 'tinyint', 3)->setDefault(0);
            });
        }
    public function upgrade113Step1()
    {
        $sm = $this->schemaManager();
        $sm->alterTable('xf_uw_comment', function(Alter $table) {
            $table->addColumn('attach_count', 'int')->setDefault(0);
            $table->addColumn('embed_metadata', 'blob')->nullable();
        });
    }

        public function uninstallStep1($params)
	{
            if(isset($params['uw_preserve']) && $params['uw_preserve'])
            {
                $comments = \XF::app()->finder('UW\FCS:Comment')
                    ->with('Post')
                    ->with('User')
                    ->where('status',1)
                    ->fetch();
                
                $posts = [];
                $users = [];
                $threadIds = [];
                foreach ( $comments as $comment )
                {
                    if($comment->Post)
                    {
                        $message = $comment->Post->getQuoteWrapper($comment->Post->message);
                        $message = $message .$comment['message'];
                        
                        $post = [];
                        $post['message'] = preg_replace('/\[CQ=.*\[\/CQ]/', '',$message);
                        $post['post_date'] =$comment['comment_date'];
                        $post['user_id'] =$comment['user_id'];
                        $post['thread_id'] =$comment['thread_id'];
                        $post['last_edit_date'] =$comment['last_edit_date'];
                        $post['username'] =$comment->username;
                        $post['position'] = 0;
                        $post['reaction_users'] = $comment->reaction_users;
                        $post['reactions'] = $comment->reactions;
                        $post['reaction_score'] = $comment->reaction_score;

                        $postEntity = \XF::app()->em()->create('XF:Post');
                        $postEntity->bulkSet($post);
                        $postEntity->save();

                        $post_id = $postEntity->post_id;

                        $update = [];
                        $update['content_type'] = 'post';
                        $update['content_id'] = $post_id;

                        \XF::db()->update('xf_reaction_content', $update, " content_type = 'post_comment' AND content_id = ".$comment['comment_id']);

    //                    $users[$comment['user_id']] = isset($users[$comment['user_id']]) ? $users[$comment['user_id']] + 1 : 1;
                        $threadIds[] = $comment['thread_id'];
                    }
                }
                
                if($threadIds)
                {
//                    \XF::db()->insertBulk('xf_post', $posts);
//                    foreach ($users as $key => $user)
//                    {
//                        $this->db()->query("
//				UPDATE xf_user
//				SET message_count = GREATEST(0, message_count + ?)
//				WHERE user_id = ?
//			", [$user, $key]);
//                    }
                    
                    $threadIds = array_unique($threadIds);
                    
                    $threads = \XF::app()->finder('XF:Thread')
                                ->where('thread_id',$threadIds)
                                ->fetch();
                                        
                    $threadRepo = \XF::app()->repository('XF:Thread');
                    
                    foreach ( $threads as $thread)
                    {
                        $threadRepo->rebuildThreadPostPositions($thread->thread_id);
                        $threadRepo->rebuildThreadUserPostCounters($thread->thread_id);
                        $thread->rebuildCounters();
                        $thread->save();
                    }
                }
            }
        }
        public function uninstallStep2($params)
	{
            $sm = $this->schemaManager();
            $sm->dropTable('xf_uw_comment');
            \XF::db()->query(" DELETE FROM xf_reaction_content WHERE content_type = 'post_comment'");
        }
        public function uninstallStep3($params)
	{
            $sm = $this->schemaManager();
            $sm->alterTable('xf_user_option', function(Alter $table) {
                $table->dropColumns(['comment_notif_all']);
            });
        }
        
        
}